      program WSOMAG
c
c-----------------------------------------------------------------------
c
c ****** Reads ASCII magnetograms of the line-of-sight-magnetic
c ****** field data from Wilcox Solar Observatory (Stanford
c ****** University), and writes the radial field to an output file.
c
c-----------------------------------------------------------------------
c
      character cname*(*),cvers*(*),cdate*(*)
c
      parameter (cname='WSOMAG')
      parameter (cvers='1.02')
      parameter (cdate='02/24/97')
c
c-----------------------------------------------------------------------
c
c ****** Updates and bug fixes:
c
c        01/31/95, ZM, Version 1.00:
c
c         - Original version of program.
c
c        08/04/95, JL, Version 1.01:
c
c         - Added option for Wang and Sheeley estimate
c           of saturation factor.  Specify 'Wilcox' for
c           uniform, 'Wang' for latitudinally varying estimate.
c
c        02/24/97, ZM, Version 1.02:
c
c         - Added a flux balance check.
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
      parameter (ntdata=30)
      parameter (nt=ntdata+6,np=73)
c
      real f(ntdata),lambda(ntdata),lambdah(ntdata+1)
c
      real theta(nt),phi(np)
      real blos(nt,np),br(nt,np),scr(nt,np)
      real fc(nt),fs(nt),satfac(nt)
c
c-----------------------------------------------------------------------
c
      character*72 fname
      character*32 oname
      character*6 satest
      character cr*4,cr0*4,long*3
c pete change
      CHARACTER*80 line_dummy
c
c-----------------------------------------------------------------------
c
      write (*,*)
      write (*,*) '### ',cname,', version ',cvers,', ',cdate,' ###'
c
      write (*,*)
      write (*,'(1x,a,$)') 'Magnetogram file name: '
      read (*,'(a)') fname
c
      write (*,*)
c
c ****** Set up the coordinate meshes.
c
      dp=2.*pi/(np-1.)
c
      do i=1,np
        phi(i)=(i-1.)*dp
      enddo
c
      dst=2./ntdata
c
      do i=1,ntdata+1
        lambdah(i)=1.-(i-1.)*dst
      enddo
c
      do i=1,ntdata
        lambda(i)=.5*(lambdah(i)+lambdah(i+1))
      enddo
c
      do i=1,ntdata
        theta(i+3)=.5*pi-asin(lambda(i))
      enddo
c
c ****** Add theta points near the poles.
c
      dt0=theta(4)/3.
c
      theta(1)=0.
      theta(2)=dt0
      theta(3)=2.*dt0
      theta(ntdata+4)=pi-2.*dt0
      theta(ntdata+5)=pi-dt0
      theta(ntdata+6)=pi
c
c ****** Initialize the BLOS array to 0.
c
      do i=1,nt
        do j=1,np
          blos(i,j)=0.
        enddo
      enddo
c
c ****** Read NP strips of data.
c
      call ffopen (1,fname,'r')
c
c ****** Pete change: New WSO format has 3 extra lines of text at the top, we need
c ****** to read them and dump them.
c
      DO i=1,3
      READ (1,'(a)') line_dummy
      WRITE(*,*) line_dummy
      ENDDO
c ****** end Pete change
c
c
      do j=1,np
c
c ****** Read the next record.
c
        call getrec (cr,long,f,ierr)
c
        if (ierr.ne.0) then
          write (*,*)
          write (*,*) '### ERROR in ',cname,':'
          write (*,*) '### Could not read a record.'
          write (*,*) 'While reading longitude: ',j
          call exit (1)
        end if
c
        if (j.eq.1) then
          write (*,*)
          write (*,*) '### Reading Carrington rotation: ',cr
          cr0=cr
        end if
c
c ****** Store the record into the buffer.
c
        do i=1,ntdata
          blos(i+3,j)=f(i)
        enddo
c
      enddo
c
c ****** Get the filter parameters.
c
      write (*,*)
      write (*,*) '### Longitudinal filter parameters:'
      write (*,*)
      write (*,*) 'Filter type: exp[-(0.7*m/MMAX)**8)]'
      write (*,*)
      write (*,*) 'MMAX specifies the mode that is attenuated to'//  ' 9
     &5% of its original amplitude.'
      write (*,*) 'Modes with M smaller than MMAX are not'//' attenuated
     & appreciably.'
      write (*,*) 'The mode with M = 1.64*MMAX is attenuated'//   ' to 5
     &% of its original amplitude.'
      write (*,*) 'If filtering is not desired, set MMAX=0.'
c
      write (*,*)
      write (*,'(1x,a,$)') 'Enter MMAX: '
      read (*,*) mmax
c
      write (*,*)
      write (*,*) ' Estimate for saturation factor '
      write (*,'(1x,a,$)') ' Enter Wilcox or Wang: '
      read (*,'(a)') satest
      write (*,*)
      if((satest .ne. 'Wilcox').and.(satest .ne. 'Wang')) then
        satest = 'Wilcox'
        write (*,*) ' satest not found, using the Wilcox estimate'
      else
        write (*,*) ' Using the ',satest,' saturation estimate'
      end if
c
c
c ****** Get the radial field.
c
      call getbr (nt,np,theta,blos,br,scr,fc,fs,mmax,satest,satfac)
c
c ****** Write the data to a text file.
c
      oname='br'//cr0//'.dat'
c
      lf=lenstr(oname)
c
      call ffopen (2,oname(1:lf),'rw')
      write (2,*) 2
      write (2,*) nt
      write (2,*) np
      write (2,*) 1
      write (2,*) (theta(i),i=1,nt)
      write (2,*) (phi(j),j=1,np)
      write (2,*) ((br(i,j),i=1,nt),j=np,1,-1)
      close (2)
c
      write (*,*)
      write (*,*) '### Wrote Br to file: ',oname(1:lf)
c
      close (1)
c
      call exit (0)
c
      end
c#######################################################################
      subroutine getrec (cr,long,f,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Get a record of 30 line-of-sight magnetic field points
c ****** versus sine-latitude from the WSO magnetogram file.
c
c ****** The file is assumed to be open, and at the location of the
c ****** record wanted.
c
c-----------------------------------------------------------------------
c
      real f(30)
      character*(*) cr,long
c
c-----------------------------------------------------------------------
c
      logical rdlin
c
c-----------------------------------------------------------------------
c
      character*80 line(4)
      character*10 id
c
c-----------------------------------------------------------------------
c
      ierr=0
c
c ****** Read four lines of data into the buffer.
c
      do i=1,4
        if (.not.rdlin(1,line(i))) then
          ierr=1
          return
        end if
      enddo
c
c ****** Extract the data.
c
      read (line(1),'(a10)') id
c
      cr=id(3:6)
      long=id(8:10)
c
      read (line(1)(19:72),'(6f9.3)') (f(i),i=1,6)
      read (line(2)( 1:72),'(8f9.3)') (f(i),i=7,14)
      read (line(3)( 1:72),'(8f9.3)') (f(i),i=15,22)
      read (line(4)( 1:72),'(8f9.3)') (f(i),i=23,30)
c
      return
      end
c#######################################################################
      subroutine getbr (nt,np,theta,blos,br,brf,fc,fs,mmax,satest,satfac
     &)
c
c-----------------------------------------------------------------------
c
      real theta(nt)
      real blos(nt,np),br(nt,np),brf(nt,np)
      real fc(nt),fs(nt),satfac(nt)
      character *(*) satest
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
c ****** Multiply the magnetic field by the saturation
c ****** factor and convert from micro-Tesla to Gauss.
c ****** The original estimate used by the Wilcox observatory,
c ****** is a uniform factor of 1.8.
c ****** Wang and Sheeley suggest 4.5 - 2.5 cos(theta)**2.
c
      if(satest .eq. 'Wang') then
        do i = 1,nt
          satfac(i) = .045 - .025*cos(theta(i))**2
        enddo
      else
        do i = 1,nt
          satfac(i) = .018
        enddo
      end if
c
      do j=1,np
        do i=1,nt
          blos(i,j) = satfac(i)*blos(i,j)
        enddo
      enddo
c
c ****** Set the radial field from the line-of-sight field.
c
      do j=1,np
        do i=4,nt-3
          br(i,j)=blos(i,j)/sin(theta(i))
        enddo
      enddo
c
c ****** Enforce periodicity at phi=0.
c
      if (mmax.gt.0) then
        do i=4,nt-3
          brav=.5*(br(i,1)+br(i,np))
          br(i, 1)=brav
          br(i,np)=brav
        enddo
      endif
c
c ****** Initialize the polar points to zero.
c
      do j=1,np
        do i=1,3
          br(i,j)=0.
        enddo
        do i=nt-2,nt
          br(i,j)=0.
        enddo
      enddo
c
c ****** Initialize the BRF array to zero.
c
      do i=1,nt
        do j=1,np
          brf(i,j)=0.
        enddo
      enddo
c
c ****** Transform the magnetic field to Fourier space, and
c ****** interpolate in the polar regions.
c
      dp=2.*pi/(np-1.)
c
      npd2=(np-1)/2
c
      write (*,*)
      write (*,*) '### Azimuthal mode amplitudes (original data):'
      write (*,*)
      write (*,*) 'm',char(9),'Amplitude'
c
      do m=0,npd2
c
        call ft (nt,np,br,m,fc,fs)
c
        if (m.eq.0) then
          call m0int (nt,theta,fc,a1,b1,a2,b2)
          do i=1,3
            fc(i)=a1+b1*theta(i)**2
            fs(i)=0.
          enddo
          do i=nt-2,nt
            fc(i)=a2+b2*(pi-theta(i))**2
            fs(i)=0.
          enddo
        else
          call mint (nt,theta,fc,m,ac1,ac2)
          call mint (nt,theta,fs,m,as1,as2)
          do i=1,3
            fc(i)=ac1*theta(i)**m
            fs(i)=as1*theta(i)**m
          enddo
          do i=nt-2,nt
            fc(i)=ac2*(pi-theta(i))**m
            fs(i)=as2*(pi-theta(i))**m
          enddo
        end if
c
c ****** Write diagnostics.
c
        fmx=0.
        do i=1,nt
          fmx=amax1(fmx,abs(fc(i)),abs(fs(i)))
        enddo
c
        write (*,*) m,char(9),fmx
c
c ****** Reconstruct the field from the Fourier coefficients.
c
        if (m.eq.0.or.m.eq.npd2) then
          fcc=1.
        else
          fcc=2.
        end if
c
        do j=1,np
c
          phi=(j-1.)*dp
c
          do i=1,nt
c
c ****** Set the filter coefficients.
c
            if (mmax.gt.0) then
              fmnorm=amax1(4.,mmax*abs(sin(theta(i))))
              filt=exp(-(.7*float(m)/fmnorm)**8)
            else
              filt=1.
            end if
c
            brf(i,j)=brf(i,j)+filt*fcc*( fc(i)*cos(m*phi)    +fs(i)*sin(
     &m*phi))
c
          enddo
c
        enddo
c
      enddo
c
      do i=1,nt
        do j=1,np
          br(i,j)=brf(i,j)
        enddo
      enddo
c
c ****** Write the mode amplitudes after filtering.
c
      if (mmax.gt.0) then
c
        write (*,*)
        write (*,*) '### Azimuthal mode amplitudes (after filter):'
        write (*,*)
        write (*,*) 'm',char(9),'Amplitude'
c
        do m=0,npd2
          call ft (nt,np,br,m,fc,fs)
          fmx=0.
          do i=1,nt
            fmx=amax1(fmx,abs(fc(i)),abs(fs(i)))
          enddo
          write (*,*) m,char(9),fmx
        enddo
c
      end if
c
c ****** Perform a flux balance check.
c
      fluxp=0.
      fluxm=0.
      do i=4,nt-3
        do j=1,np-1
          dtheta=.5*(theta(i+1)-theta(i-1))
          da=sin(theta(i))*dtheta*dp
          if (br(i,j).ge.0.) then
            fluxp=fluxp+br(i,j)*da
          else
            fluxm=fluxm+br(i,j)*da
          end if
        enddo
      enddo
c
      write (*,*)
      write (*,*) '### Flux balance (WSO data points):'
      write (*,*)
      write (*,*) 'Positive flux = ',fluxp
      write (*,*) 'Negative flux = ',fluxm
c
      fluxp=0.
      fluxm=0.
      do i=2,nt-1
        do j=1,np-1
          dtheta=.5*(theta(i+1)-theta(i-1))
          da=sin(theta(i))*dtheta*dp
          if (br(i,j).ge.0.) then
            fluxp=fluxp+br(i,j)*da
          else
            fluxm=fluxm+br(i,j)*da
          end if
        enddo
      enddo
c
      write (*,*)
      write (*,*) '### Flux balance (total):'
      write (*,*)
      write (*,*) 'Positive flux = ',fluxp
      write (*,*) 'Negative flux = ',fluxm
c
      return
      end
c#######################################################################
      subroutine m0int (nt,t,f,a1,b1,a2,b2)
c
c-----------------------------------------------------------------------
c
c ****** Get the coefficients of the variation of the m = 0
c ****** mode near the poles.
c
c ****** This assumes that the m = 0 component varies as
c ****** ( a + b * theta**2 ) near the poles.
c
c-----------------------------------------------------------------------
c
      real t(nt),f(nt)
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
      b1=(f(5)-f(4))/(t(5)**2-t(4)**2)
      a1=f(4)-b1*t(4)**2
c
      b2=(f(nt-4)-f(nt-3))/((pi-t(nt-4))**2-(pi-t(nt-3))**2)
      a2=f(nt-3)-b2*(pi-t(nt-3))**2
c
      return
      end
c#######################################################################
      subroutine mint (nt,t,f,m,a1,a2)
c
c-----------------------------------------------------------------------
c
c ****** Get the coefficients of the variation of the m > 0
c ****** modes near the poles.
c
c ****** This assumes that the m > 0 components vary as
c ****** ( a * theta**m ) near the poles.
c
c-----------------------------------------------------------------------
c
      real t(nt),f(nt)
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
      a1=f(4)/t(4)**m
c
      a2=f(nt-3)/(pi-t(nt-3))**m
c
      return
      end
c#######################################################################
      subroutine ft (nt,np,f,m,fc,fs)
c
c-----------------------------------------------------------------------
c
c ****** Take the Fourier transform of the array F for all NT points.
c
c-----------------------------------------------------------------------
c
      real f(nt,np),fs(nt),fc(nt)
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
      dp=2.*pi/(np-1.)
c
      fnorm=1./(np-1.)
c
      do i=1,nt
        fs(i)=0.
        fc(i)=0.
        do j=1,np-1
          phi=(j-1.)*dp
          fs(i)=fs(i)+sin(m*phi)*f(i,j)
          fc(i)=fc(i)+cos(m*phi)*f(i,j)
        enddo
        fs(i)=fnorm*fs(i)
        fc(i)=fnorm*fc(i)
      enddo
c
      return
      end
c#######################################################################
      function lenstr (str)
c
c-----------------------------------------------------------------------
c
c ****** Get the length of string STR, ignoring right-filling blanks.
c
c-----------------------------------------------------------------------
c
      character*(*) str
c
c-----------------------------------------------------------------------
c
      do 100 i=len(str),1,-1
        if (str(i:i).ne.' ') go to 200
  100 continue
      i=1
  200 continue
c
      lenstr=i
c
      return
      end
c#######################################################################
      subroutine ffopen (iun,fname,mode)
c
c-----------------------------------------------------------------------
c
c ****** Open file FNAME and link it to unit IUN.
c
c-----------------------------------------------------------------------
c
c ****** When MODE='r', the file must exist.
c ****** When MODE='w', the file is created.
c ****** When MODE='rw', the file must exist, but can be overwritten.
c
c-----------------------------------------------------------------------
c
      character*(*) fname,mode
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (mode.eq.'r') then
        open (iun,file=fname(1:lf),status='old',err=900)
      else if (mode.eq.'rw') then
        open (iun,file=fname(1:lf),err=900)
      else if (mode.eq.'w') then
        open (iun,file=fname(1:lf),status='new',err=900)
      else
        write (*,*)
        write (*,*) '### ERROR in FFOPEN:'
        write (*,*) '### Invalid MODE requested.'
        write (*,*) 'MODE = ',mode
        write (*,*) 'File name = ',fname(1:lf)
        call exit (1)
      end if
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in FFOPEN:'
      write (*,*) '### Error while opening file.'
      write (*,*) 'File name = ',fname(1:lf)
      write (*,*) 'MODE =  ',mode
      call exit (1)
      end
c#######################################################################
      logical function rdlin (iun,line)
c
c-----------------------------------------------------------------------
c
c ****** Read a line of text from unit IUN.
c
c ****** If a line was read successfully, return with
c ****** RDLIN=.true.; otherwise, set RDLIN=.false.
c
c-----------------------------------------------------------------------
c
      character*(*) line
c
c-----------------------------------------------------------------------
c
      read (iun,'(a)',end=100,err=100) line
c
      rdlin=.true.
      return
c
  100 continue
c
c ****** Error while reading the line.
c
      rdlin=.false.
c
      return
      end
