;=======================================================================
;+
; NAME:
;    rdr
; PURPOSE:
;    Retrieve data from a NetCDF file
; CATEGORY:
;    Input/Output
; CALLING SEQUENCE:
;    rdr [, filename [, varname [, data ]]]
; INPUTS:
;    filename:  Name of a NetCDF file (string or alphabetic no.)
;    varname:   Name of a variable (string or alphabetic no.)
; INPUT KEYWORD PARAMETERS:
;    offset:    Starting positions for the read
;               (default is [0,0,0,...])
;    count:     Counts to be used in reading value
;               (default is to read all elements of variable)
;    directory: Directory containing files
;               (default is current working directory)
; OUTPUTS:
;    data:      Values of a variable
; OUTPUT KEYWORD PARAMETERS:
;    None
; COMMON BLOCKS:
;    None
; EXAMPLE:
;    rdr
;               (names of all files in the directory)
;    rdr,'run.nc'
;               (details of "run.nc" file)
;    rdr,4
;               (details of 4th file)
;    rdr,'run.nc','b1'
;               (details of "b1" variable from "run.nc" file)
;    rdr,4,6
;               (details of 6th variable from 4th file)
;    rdr,'run.nc','b1',b1
;               (values of "b1" variable from "run.nc" file)
;    rdr,4,6,b1
;               (values of 6th variable from 4th file)
; MODIFICATION HISTORY:
;    D. Odstrcil, Ondrejov, Jan 1996
;-
;=======================================================================
pro rdr, filename, varname, data, offset=offset, $
  count=count, directory=directory
;=======================================================================
;DIRECTORY
;-----------------------------------------------------------------------
;Directory
  cd,current=owd
  if(n_elements(directory) gt 0) then cd, directory
  cd,current=cwd
;-----------------------------------------------------------------------
;Gain names of all files within the directory
  lfile=findfile('*.nc')
  nfile=n_elements(lfile)
;-----------------------------------------------------------------------
;Exit if no files
  if(nfile eq 0) then begin
    print,'*** no NetCDF files in directory: '+cwd
    goto,return
  endif
;=======================================================================
;LIST ALL FILES
;-----------------------------------------------------------------------
  iarg=n_params()
  if(iarg gt 0) then goto, file
;-----------------------------------------------------------------------
;Directory
  print,'=== directory:  '+cwd
;-----------------------------------------------------------------------
;List all files
  print,'--- files:'
  lfff=strarr(nfile)
  for jfile=0,nfile-1 do begin
    i16='                '
    case 1 of
      ((jfile ge 000) and (jfile le 009)): begin
        strput,i16,strtrim(string(jfile),2)+'=',3
      end
      ((jfile ge 010) and (jfile le 099)): begin
        strput,i16,strtrim(string(jfile),2)+'=',2
      end
      ((jfile ge 100) and (jfile le 999)): begin
        strput,i16,strtrim(string(jfile),2)+'=',1
      end
    endcase
    strput,i16,lfile(jfile),5
    lfff(jfile)=i16
  endfor
  print,format='(5(a))',lfff
;-----------------------------------------------------------------------
;Exit
  goto, return
;=======================================================================
;FILE
file:
;-----------------------------------------------------------------------
;File name
  ifile=size(filename)
  if(ifile(1) eq 2) then filename=lfile(filename)
;-----------------------------------------------------------------------
;Open the file
  fileid=ncdf_open(filename,/nowrite)
;-----------------------------------------------------------------------
;File details
  fileinfo=ncdf_inquire(fileid)
;=======================================================================
;FILE INFORMATION
  if(iarg gt 1) then goto, variable
;-----------------------------------------------------------------------
;File
  print,'=== file      :  '+filename
  print,format='(a15,2x,a)','directory =',cwd
;-----------------------------------------------------------------------
;Global attributes
  if(fileinfo.ngatts eq 0) then begin
    print,'--- attributes: none'
  endif else begin
    print,'--- attributes:'
    for jattnum=0,fileinfo.ngatts-1 do begin
      attname=ncdf_attname(fileid,jattnum,/global)
      ncdf_attget,fileid,attname,attvalue,/global
      print,format='(a15,2x,a)',attname+' =',string(attvalue)
    endfor
  endelse
;-----------------------------------------------------------------------
;Dimensions
  if(fileinfo.ndims eq 0) then begin
    print,'--- dimensions: none'
  endif else begin
    for jdims=0,fileinfo.ndims-1 do begin
      ncdf_diminq,fileid,jdims,name,size
      if(jdims eq fileinfo.recdim) then begin
        print,format='(a15,i6,a13)',name+' =',size,'  (unlimited)'
      endif else begin
        print,format='(a15,i6)',name+' =',size
      endelse
    endfor
  endelse
;-----------------------------------------------------------------------
;Variables
  if(fileinfo.nvars eq 0) then begin
    print,'*** variables: none'
    goto,close
  endif else begin
    print,'--- variables:'
    lfff=strarr(fileinfo.nvars)
    for jvar=0,fileinfo.nvars-1 do begin
      i16='                '
      if (jvar lt 10) then begin
        strput,i16,strtrim(string(jvar),2),5
      endif else begin
        strput,i16,strtrim(string(jvar),2),4
      endelse
      varinfo=ncdf_varinq(fileid,jvar)
      strput,i16,varinfo.name,8
      lfff(jvar)=i16
    endfor
    print,format='(4(a))',lfff
  endelse
;-----------------------------------------------------------------------
;Exit
  goto, close
;=======================================================================
;VARIABLE
variable:
;-----------------------------------------------------------------------
;Variable name and information
  ivar=size(varname)
  if(ivar(1) eq 2) then begin
    varid=varname
    varinfo=ncdf_varinq(fileid,varid)
    varname=varinfo.name
  endif else begin
    varid=ncdf_varid(fileid,varname)
    varinfo=ncdf_varinq(fileid,varid)
  endelse
;=======================================================================
;VARIABLE INFORMATION
  if(iarg gt 2) then goto, data
;-----------------------------------------------------------------------
;Variable
  print,'=== variable  :  '+varname
  print,format='(a15,2x,a)','file =',filename
  print,format='(a15,2x,a)','directory =',cwd
;-----------------------------------------------------------------------
;Attributes
  print,'--- attributes:'
  for jattnum=0,varinfo.natts-1 do begin
    attname=ncdf_attname(fileid,varid,jattnum)
    ncdf_attget,fileid,varid,attname,attvalue
    print,format='(a15,2x,a)',attname+' =',string(attvalue)
  endfor
;-----------------------------------------------------------------------
;Dimensions
  print,'--- dimensions:'
  for jdim=0,varinfo.ndims-1 do begin
    ncdf_diminq,fileid,varinfo.dim(jdim),dimname,dimsize
    print,format='(a15,2x,i6)',dimname+' =',dimsize
  endfor
;-----------------------------------------------------------------------
;Exit
  goto, close
;=======================================================================
;DATA
data:
;-----------------------------------------------------------------------
;Get a hyperslab of values from a stored variable
  if((n_elements(offset) ne 0) and $
    (n_elements(count) ne 0)) then begin
    ncdf_varget,fileid,varid,data,offset=offset,count=count
  endif else begin
    ncdf_varget,fileid,varid,data
  endelse
;-----------------------------------------------------------------------
;Determine all dimensions
  isize=size(data)
  if(isize(0) ge 1) then i1=isize(1) else i1=0
  if(isize(0) ge 2) then i2=isize(2) else i2=0
  if(isize(0) ge 3) then i3=isize(3) else i3=0
  if(isize(0) ge 4) then i4=isize(4) else i4=0
;-----------------------------------------------------------------------
;Eliminate unit dimensions
  if(i1 gt 0) then data=float(reform(data))
;-----------------------------------------------------------------------
;Variable
  print,'=== variable  :  '+varname,i1,i2,i3,i4
  print,format='(a15,2x,a)','file =',filename
  print,format='(a15,2x,a)','directory =',cwd
;=======================================================================
;EXIT
;-----------------------------------------------------------------------
;Close the file
close:
  ncdf_close,fileid
  if(cwd ne owd) then cd, owd
;-----------------------------------------------------------------------
return:
end
